#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)


class ResourceDataAvro:
    # Use __slots__ to explicitly declare all data members.
    __slots__ = ["request_properties", "config_properties", "resource_properties"]

    def __init__(self):
        self.request_properties = None
        self.config_properties = None
        self.resource_properties = None

    def from_dict(self, value: dict):
        self.request_properties = value.get('request_properties', None)
        self.config_properties = value.get('config_properties', None)
        self.resource_properties = value.get('resource_properties', None)

    def to_dict(self) -> dict:
        """
            The Avro Python library does not support code generation.
            For this reason we must provide a dict representation of our class for serialization.
        """
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {}
        if self.config_properties is not None and len(self.config_properties) > 0:
            result['config_properties'] = self.config_properties

        if self.request_properties is not None and len(self.request_properties) > 0:
            result['request_properties'] = self.request_properties

        if self.resource_properties is not None and len(self.resource_properties) > 0:
            result['resource_properties'] = self.resource_properties

        return result

    def __str__(self):
        return "request_properties: {} config_properties: {} resource_properties: {}"\
            .format(self.request_properties, self.config_properties, self.resource_properties)

    def __eq__(self, other):
        if not isinstance(other, ResourceDataAvro):
            return False

        return self.request_properties == other.request_properties and \
               self.config_properties == other.config_properties and \
               self.resource_properties == other.resource_properties

    def validate(self) -> bool:
        ret_val = True
        return ret_val


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import unittest
from datetime import datetime

from fabric.message_bus.admin import AdminApi
from fabric.message_bus.consumer import AvroConsumerApi
from fabric.message_bus.messages.actor_avro import ActorAvro
from fabric.message_bus.messages.add_reservation_avro import AddReservationAvro
from fabric.message_bus.messages.add_reservations_avro import AddReservationsAvro
from fabric.message_bus.messages.add_slice_avro import AddSliceAvro
from fabric.message_bus.messages.auth_avro import AuthAvro
from fabric.message_bus.messages.claim_avro import ClaimAvro
from fabric.message_bus.messages.claim_delegation_avro import ClaimDelegationAvro
from fabric.message_bus.messages.claim_resources_avro import ClaimResourcesAvro
from fabric.message_bus.messages.close_avro import CloseAvro
from fabric.message_bus.messages.close_reservations_avro import CloseReservationsAvro
from fabric.message_bus.messages.delegation_avro import DelegationAvro
from fabric.message_bus.messages.demand_reservation_avro import DemandReservationAvro
from fabric.message_bus.messages.extend_lease_avro import ExtendLeaseAvro
from fabric.message_bus.messages.extend_reservation_avro import ExtendReservationAvro
from fabric.message_bus.messages.extend_ticket_avro import ExtendTicketAvro
from fabric.message_bus.messages.failed_rpc_avro import FailedRPCAvro
from fabric.message_bus.messages.get_actors_avro import GetActorsAvro
from fabric.message_bus.messages.get_pool_info_avro import GetPoolInfoAvro
from fabric.message_bus.messages.get_reservation_units_avro import GetReservationUnitsAvro
from fabric.message_bus.messages.get_reservations_request_avro import GetReservationsRequestAvro
from fabric.message_bus.messages.get_reservations_state_request_avro import GetReservationsStateRequestAvro
from fabric.message_bus.messages.get_unit_avro import GetUnitAvro
from fabric.message_bus.messages.modify_lease_avro import ModifyLeaseAvro
from fabric.message_bus.messages.pool_info_avro import PoolInfoAvro
from fabric.message_bus.messages.proxy_avro import ProxyAvro
from fabric.message_bus.messages.reservation_state_avro import ReservationStateAvro
from fabric.message_bus.messages.result_actor_avro import ResultActorAvro
from fabric.message_bus.messages.result_pool_info_avro import ResultPoolInfoAvro
from fabric.message_bus.messages.result_proxy_avro import ResultProxyAvro
from fabric.message_bus.messages.result_reservation_avro import ResultReservationAvro
from fabric.message_bus.messages.get_slices_request_avro import GetSlicesRequestAvro
from fabric.message_bus.messages.result_reservation_state_avro import ResultReservationStateAvro
from fabric.message_bus.messages.result_slice_avro import ResultSliceAvro
from fabric.message_bus.messages.query_avro import QueryAvro
from fabric.message_bus.messages.query_result_avro import QueryResultAvro
from fabric.message_bus.messages.redeem_avro import RedeemAvro
from fabric.message_bus.messages.remove_reservation_avro import RemoveReservationAvro
from fabric.message_bus.messages.remove_slice_avro import RemoveSliceAvro
from fabric.message_bus.messages.reservation_avro import ReservationAvro
from fabric.message_bus.messages.reservation_mng import ReservationMng
from fabric.message_bus.messages.resource_data_avro import ResourceDataAvro
from fabric.message_bus.messages.resource_set_avro import ResourceSetAvro
from fabric.message_bus.messages.result_avro import ResultAvro
from fabric.message_bus.messages.result_string_avro import ResultStringAvro
from fabric.message_bus.messages.result_strings_avro import ResultStringsAvro
from fabric.message_bus.messages.result_unit_avro import ResultUnitAvro
from fabric.message_bus.messages.slice_avro import SliceAvro
from fabric.message_bus.messages.term_avro import TermAvro
from fabric.message_bus.messages.ticket_avro import TicketAvro
from fabric.message_bus.messages.ticket_reservation_avro import TicketReservationAvro
from fabric.message_bus.messages.unit_avro import UnitAvro
from fabric.message_bus.messages.update_data_avro import UpdateDataAvro
from fabric.message_bus.messages.update_delegation_avro import UpdateDelegationAvro
from fabric.message_bus.messages.update_lease_avro import UpdateLeaseAvro
from fabric.message_bus.messages.update_reservation_avro import UpdateReservationAvro
from fabric.message_bus.messages.update_slice_avro import UpdateSliceAvro
from fabric.message_bus.messages.update_ticket_avro import UpdateTicketAvro
from fabric.message_bus.messages.message import IMessageAvro
from fabric.message_bus.producer import AvroProducerApi


class MessageBusTest(unittest.TestCase):
    def test_consumer_producer(self):
        from confluent_kafka import avro
        from threading import Thread
        import time

        conf = {'metadata.broker.list': 'localhost:19092',
                'security.protocol': 'SSL',
                'group.id': 'ssl-host',
                'ssl.ca.location': '../../../secrets/snakeoil-ca-1.crt',
                'ssl.key.location': '../../../secrets/kafkacat.client.key',
                'ssl.key.password': 'confluent',
                'ssl.certificate.location': '../../../secrets/kafkacat-ca1-signed.pem'
        }
        # Create Admin API object
        api = AdminApi(conf)

        for a in api.list_topics():
            print("Topic {}".format(a))

        topics = ['fabric-mb-public-test1', 'fabric-mb-public-test2']

        # create topics
        api.delete_topics(topics)
        api.create_topics(topics, num_partitions=1, replication_factor=1)

        # load AVRO schema
        file = open('../schema/key.avsc', "r")
        key_bytes = file.read()
        file.close()
        key_schema = avro.loads(key_bytes)
        file = open('../schema/message.avsc', "r")
        val_bytes = file.read()
        file.close()
        val_schema = avro.loads(val_bytes)

        conf['schema.registry.url']="http://localhost:8081"

        # create a producer
        producer = AvroProducerApi(conf, key_schema, val_schema)

        # push messages to topics

        id_token = 'id_token'
        # udd
        udd = UpdateDataAvro()
        udd.message = "message"
        udd.failed = False

        auth = AuthAvro()
        auth.guid = "testguid"
        auth.name = "testactor"

        # Query
        query = QueryAvro()
        query.message_id = "msg1"
        query.callback_topic = "topic"
        query.properties = {"abc": "def"}
        query.auth = auth
        query.id_token = id_token
        #print(query.to_dict())
        producer.produce_sync("fabric-mb-public-test1", query)

        # QueryResult
        query_result = QueryResultAvro()
        query_result.message_id = "msg2"
        query_result.request_id = "req2"
        query_result.properties = {"abc": "def"}
        query_result.auth = auth
        #print(query_result.to_dict())
        producer.produce_sync("fabric-mb-public-test2", query_result)

        # FailedRPC
        failed_rpc = FailedRPCAvro()
        failed_rpc.message_id = "msg3"
        failed_rpc.request_id = "req3"
        failed_rpc.reservation_id = "rsv_abc"
        failed_rpc.request_type = 1
        failed_rpc.error_details = "test error message"
        failed_rpc.auth = auth
        #print(failed_rpc.to_dict())
        producer.produce_sync("fabric-mb-public-test2", failed_rpc)

        claim_req = ClaimResourcesAvro()
        claim_req.guid = "dummy-guid"
        claim_req.auth = auth
        claim_req.broker_id = "brokerid"
        claim_req.reservation_id = "rsv_id"
        claim_req.delegation_id = "dlg_id"
        claim_req.message_id = "test_claim_1"
        claim_req.callback_topic = "test"
        claim_req.slice_id = "slice_1"
        claim_req.id_token = id_token

        #print(claim_req.to_dict())
        producer.produce_sync("fabric-mb-public-test2", claim_req)
        
        reservation = ReservationAvro()
        reservation.reservation_id = "res123"
        reservation.sequence = 1
        reservation.slice = SliceAvro()
        reservation.slice.guid = "slice-12"
        reservation.slice.slice_name = "test_slice"
        reservation.slice.description = "test description"
        reservation.slice.owner = auth
        reservation.term = TermAvro()
        reservation.term.start_time = 1593854111999
        reservation.term.end_time = 1593854111999
        reservation.term.new_start_time = 1593854111999
        reservation.resource_set = ResourceSetAvro()
        reservation.resource_set.units = 0
        reservation.resource_set.type = "type1"
        reservation.resource_set.resource_data = ResourceDataAvro()
        reservation.resource_set.resource_data.request_properties = {'type': 'site.vlan', 'label': 'Net AM',
                                                                     'attributescount': '1',
                                                                     'attribute.0.key': 'resource.class.invfortype',
                                                                     'resource.class.invfortype.type': '6',
                                                                     'resource.class.invfortype.value': 'actor.core.policy.SimplerUnitsInventory.SimplerUnitsInventory',
                                                                     'pool.name': 'Net AM'}

        reservation.resource_set.concrete = b'\x80\x04\x95\xb9\x02\x00\x00\x00\x00\x00\x00\x8c\x16actor.core.core.Ticket\x94\x8c\x06Ticket\x94\x93\x94)\x81\x94}\x94(\x8c\tauthority\x94\x8c,actor.core.proxies.kafka.KafkaAuthorityProxy\x94\x8c\x13KafkaAuthorityProxy\x94\x93\x94)\x81\x94}\x94(\x8c\nproxy_type\x94\x8c\x05kafka\x94\x8c\x08callback\x94\x89\x8c\nactor_name\x94\x8c\x0cfabric-vm-am\x94\x8c\nactor_guid\x94\x8c\x12actor.core.util.ID\x94\x8c\x02ID\x94\x93\x94)\x81\x94}\x94\x8c\x02id\x94\x8c\x11fabric-vm-am-guid\x94sb\x8c\x04auth\x94\x8c\x18actor.security.AuthToken\x94\x8c\tAuthToken\x94\x93\x94)\x81\x94}\x94(\x8c\x04name\x94h\x0f\x8c\x04guid\x94h\x14ub\x8c\x0bkafka_topic\x94\x8c\x12fabric-vm-am-topic\x94\x8c\x04type\x94K\x03\x8c\x10bootstrap_server\x94\x8c\x0elocalhost:9092\x94\x8c\x0fschema_registry\x94\x8c\x15http://localhost:8081\x94\x8c\x0fkey_schema_file\x94\x8cK/Users/komalthareja/renci/code/fabric/ActorBase/message_bus/schema/key.avsc\x94\x8c\x11value_schema_file\x94\x8cO/Users/komalthareja/renci/code/fabric/ActorBase/message_bus/schema/message.avsc\x94ub\x8c\x0fresource_ticket\x94N\x8c\told_units\x94K\x0fub.'

        claim = ClaimAvro()
        claim.auth = auth
        claim.message_id = "msg4"
        claim.callback_topic = "test"
        claim.reservation = reservation
        #print(claim.to_dict())
        producer.produce_sync("fabric-mb-public-test2", claim)

        delegation = DelegationAvro()
        delegation.delegation_id = "dlg123"
        delegation.sequence = 1
        delegation.slice = SliceAvro()
        delegation.slice.guid = "slice-12"
        delegation.slice.slice_name = "test_slice"
        delegation.slice.description = "test description"
        delegation.slice.owner = auth

        claimd = ClaimDelegationAvro()
        claimd.auth = auth
        claimd.message_id = "msg4"
        claimd.callback_topic = "test"
        claimd.delegation = delegation
        claimd.id_token = id_token
        #print(claim.to_dict())
        producer.produce_sync("fabric-mb-public-test2", claimd)

        # Redeem
        redeem = RedeemAvro()
        redeem.message_id = "msg4"
        redeem.callback_topic = "test"
        redeem.reservation = reservation
        redeem.auth = auth
        #print(redeem.to_dict())
        producer.produce_sync("fabric-mb-public-test2", redeem)

        update_ticket = UpdateTicketAvro()
        update_ticket.auth = auth
        update_ticket.message_id = "msg11"
        update_ticket.callback_topic = "test"
        update_ticket.reservation = reservation
        update_ticket.update_data = UpdateDataAvro()
        update_ticket.update_data.failed = False
        update_ticket.update_data.message = ""

        #print(update_ticket.to_dict())
        producer.produce_sync("fabric-mb-public-test2", update_ticket)

        update_d = UpdateDelegationAvro()
        update_d.auth = auth
        update_d.message_id = "msg11"
        update_d.callback_topic = "test"
        update_d.delegation = delegation
        update_d.update_data = UpdateDataAvro()
        update_d.update_data.failed = False
        update_d.update_data.message = ""
        update_d.id_token = id_token

        #print(update_ticket.to_dict())
        producer.produce_sync("fabric-mb-public-test2", update_d)

        get_slice = GetSlicesRequestAvro()
        get_slice.auth = auth
        get_slice.message_id = "msg11"
        get_slice.callback_topic = "test"
        get_slice.guid = "guid"
        get_slice.id_token = id_token

        #print(get_slice.to_dict())

        producer.produce_sync("fabric-mb-public-test2", get_slice)

        result = ResultAvro()
        result.code = 0

        slice_res = ResultSliceAvro()
        slice_res.message_id = "msg11"
        slice_res.status = result

        s1 = SliceAvro()
        s1.set_slice_name("abc")
        s1.set_slice_id("11111")
        s1.set_owner(auth)
        s1.set_description("abcd")
        prop = {}
        s1.set_config_properties(prop)
        s1.set_resource_type('site.vm')
        s1.set_client_slice(False)
        s1.set_broker_client_slice(False)

        slice_res.slices = []
        slice_res.slices.append(s1)

        #print(slice_res.to_dict())

        producer.produce_sync("fabric-mb-public-test2", slice_res)

        res_req = GetReservationsRequestAvro()
        res_req.message_id = "abc123"
        res_req.callback_topic = "test"
        res_req.guid = "guid"
        res_req.reservation_id = "res123"
        res_req.reservation_type = "broker"
        res_req.auth = auth
        res_req.id_token = id_token

        print(res_req.to_dict())

        producer.produce_sync("fabric-mb-public-test2", res_req)

        res = ReservationMng()
        res.reservation_id = "abcd123"
        res.rtype = 'site.baremetalce'
        res.notices = 'noice'
        res.slice_id = "slice_1"
        res.start = 1264827600000
        res.end = 1927515600000
        res.requested_end = 1927515600000
        res.state = 2
        res.pending_state = 1

        res_list = [res]

        res_res = ResultReservationAvro()
        res_res.message_id = res_req.message_id
        res_res.status = result
        res_res.reservations = res_list

        #print(res_res.to_dict())

        producer.produce_sync("fabric-mb-public-test2", res_res)

        remove_slice = RemoveSliceAvro()
        remove_slice.message_id = "msg1"
        remove_slice.guid = 'guid1'
        remove_slice.slice_id = 'slice1'
        remove_slice.callback_topic = 'test_topic'
        remove_slice.auth = auth
        #print(remove_slice.to_dict())

        producer.produce_sync("fabric-mb-public-test2", remove_slice)

        status_resp = ResultStringAvro()
        status_resp.message_id = "msg1"
        status_resp.result = "abc"
        status_resp.status = result

        producer.produce_sync("fabric-mb-public-test2", status_resp)

        add_slice = AddSliceAvro()
        add_slice.message_id = "msg1"
        add_slice.guid = 'guid1'
        add_slice.slice_obj = s1
        add_slice.callback_topic = 'test_topic'
        add_slice.auth = auth
        # print(add_slice.to_dict())

        producer.produce_sync("fabric-mb-public-test2", add_slice)

        update_slice = UpdateSliceAvro()
        update_slice.message_id = "msg1"
        update_slice.guid = 'guid1'
        update_slice.slice_obj = s1
        update_slice.callback_topic = 'test_topic'
        update_slice.auth = auth
        # print(update_slice.to_dict())

        producer.produce_sync("fabric-mb-public-test2", update_slice)

        remove_res = RemoveReservationAvro()
        remove_res.message_id = "msg1"
        remove_res.guid = 'guid1'
        remove_res.reservation_id = 'rid1'
        remove_res.callback_topic = 'test_topic'
        remove_res.auth = auth
        # print(remove_res.to_dict())

        producer.produce_sync("fabric-mb-public-test2", remove_res)

        close_res = CloseReservationsAvro()
        close_res.message_id = "msg1"
        close_res.guid = 'guid1'
        close_res.reservation_id = 'rid1'
        close_res.callback_topic = 'test_topic'
        close_res.auth = auth
        # print(close_res.to_dict())

        producer.produce_sync("fabric-mb-public-test2", close_res)

        update_res = UpdateReservationAvro()
        update_res.message_id = "msg1"
        update_res.guid = 'guid1'
        update_res.reservation_obj = res
        update_res.callback_topic = 'test_topic'
        update_res.auth = auth
        print(update_res.to_dict())

        producer.produce_sync("fabric-mb-public-test2", update_res)

        ticket = TicketReservationAvro()
        ticket.reservation_id = "abcd123"
        ticket.rtype = 'site.baremetalce'
        ticket.notices = 'noice'
        ticket.slice_id = "slice_1"
        ticket.start = 1264827600000
        ticket.end = 1927515600000
        ticket.requested_end = 1927515600000
        ticket.state = 2
        ticket.pending_state = 1
        ticket.broker = "broker1"

        add_res = AddReservationAvro()
        add_res.message_id = "msg1"
        add_res.guid = 'guid1'
        add_res.reservation_obj = ticket
        add_res.callback_topic = 'test_topic'
        add_res.auth = auth
        print(add_res.to_dict())

        producer.produce_sync("fabric-mb-public-test2", add_res)

        add_ress = AddReservationsAvro()
        add_ress.message_id = "msg1"
        add_ress.guid = 'guid1'
        add_ress.reservation_list = []
        add_ress.reservation_list.append(ticket)
        add_ress.callback_topic = 'test_topic'
        add_ress.auth = auth
        print(add_ress.to_dict())

        producer.produce_sync("fabric-mb-public-test2", add_ress)

        demand_res = DemandReservationAvro()
        demand_res.message_id = "msg1"
        demand_res.guid = 'guid1'
        demand_res.reservation_obj = res
        demand_res.callback_topic = 'test_topic'
        demand_res.auth = auth
        print(demand_res.to_dict())

        producer.produce_sync("fabric-mb-public-test2", demand_res)

        extend_res = ExtendReservationAvro()
        extend_res.message_id = "msg1"
        extend_res.guid = 'guid1'
        extend_res.callback_topic = 'test_topic'
        extend_res.auth = auth
        extend_res.reservation_id = "rid1"
        extend_res.new_units = -1
        extend_res.new_resource_type = "abc"
        extend_res.request_properties = {'abcd':'eee'}
        extend_res.config_properties = {'abcd':'eee'}
        extend_res.end_time = int(datetime.now().timestamp())
        print(extend_res.to_dict())

        producer.produce_sync("fabric-mb-public-test2", extend_res)

        close = CloseAvro()
        close.message_id = "msg1"
        close.reservation = reservation
        close.callback_topic = "topic1"
        close.auth = auth

        producer.produce_sync("fabric-mb-public-test2", close)

        extend_lease = ExtendLeaseAvro()
        extend_lease.message_id = "msg1"
        extend_lease.reservation = reservation
        extend_lease.callback_topic = "topic1"
        extend_lease.auth = auth

        producer.produce_sync("fabric-mb-public-test2", extend_lease)

        extend_ticket = ExtendTicketAvro()
        extend_ticket.message_id = "msg1"
        extend_ticket.reservation = reservation
        extend_ticket.callback_topic = "topic1"
        extend_ticket.auth = auth

        producer.produce_sync("fabric-mb-public-test2", extend_ticket)

        modify_lease = ModifyLeaseAvro()
        modify_lease.message_id = "msg1"
        modify_lease.reservation = reservation
        modify_lease.callback_topic = "topic1"
        modify_lease.auth = auth

        producer.produce_sync("fabric-mb-public-test2", modify_lease)

        update_lease = UpdateLeaseAvro()
        update_lease.message_id = "msg1"
        update_lease.reservation = reservation
        update_lease.callback_topic = "topic1"
        update_lease.update_data = UpdateDataAvro()
        update_lease.update_data.failed = False
        update_lease.update_data.message = "success"
        update_lease.auth = auth

        producer.produce_sync("fabric-mb-public-test2", update_lease)

        ticket = TicketAvro()
        ticket.message_id = "msg1"
        ticket.reservation = reservation
        ticket.callback_topic = "topic1"
        ticket.auth = auth

        producer.produce_sync("fabric-mb-public-test2", ticket)

        res_state_req = GetReservationsStateRequestAvro()
        res_state_req.guid = "gud1"
        res_state_req.message_id = "msg1"
        res_state_req.reservation_ids = []
        res_state_req.reservation_ids.append("a1")
        res_state_req.callback_topic = "topic1"
        res_state_req.auth = auth
        res_state_req.id_token = id_token

        producer.produce_sync("fabric-mb-public-test2", res_state_req)

        res_strings = ResultStringsAvro()
        res_strings.status = result
        res_strings.message_id = "msg1"
        res_strings.result = []
        res_strings.result.append("r1")


        producer.produce_sync("fabric-mb-public-test2", res_strings)

        res_state = ResultReservationStateAvro()
        res_state.status = result
        res_state.message_id = "msg1"
        res_state.reservation_states = []
        ss = ReservationStateAvro()
        ss.state = 1
        ss.pending_state = 2
        res_state.reservation_states.append(ss)

        producer.produce_sync("fabric-mb-public-test2", res_state)

        ru = GetReservationUnitsAvro()
        ru.message_id = "msg1"
        ru.reservation_id = "rid1"
        ru.guid = "gud1"
        ru.message_id = "msg1"
        ru.auth = auth
        ru.callback_topic = "test"
        ru.id_token = id_token

        producer.produce_sync("fabric-mb-public-test2", ru)

        ruu = GetUnitAvro()
        ruu.message_id = "msg1"
        ruu.unit_id = "uid1"
        ruu.guid = "gud1"
        ruu.message_id = "msg1"
        ruu.auth = auth
        ruu.callback_topic = "test"
        ruu.id_token = id_token

        producer.produce_sync("fabric-mb-public-test2", ruu)

        pool_info = GetPoolInfoAvro()
        pool_info.message_id = "msg1"
        pool_info.guid = "gud1"
        pool_info.message_id = "msg1"
        pool_info.auth = auth
        pool_info.callback_topic = "test"
        pool_info.broker_id = "broker11"
        pool_info.id_token = id_token

        producer.produce_sync("fabric-mb-public-test2", pool_info)

        actors_req = GetActorsAvro()
        actors_req.message_id = "msg1"
        actors_req.guid = "gud1"
        actors_req.message_id = "msg1"
        actors_req.auth = auth
        actors_req.callback_topic = "test"
        actors_req.type = "Broker"
        actors_req.id_token = id_token

        producer.produce_sync("fabric-mb-public-test2", actors_req)

        result_unit = ResultUnitAvro()
        unit = UnitAvro()
        unit.properties = {'test':'value'}
        result_unit.message_id = "msg1"
        result_unit.status = result
        result_unit.units = []
        result_unit.units.append(unit)

        producer.produce_sync("fabric-mb-public-test2", result_unit)

        result_proxy = ResultProxyAvro()
        proxy = ProxyAvro()
        proxy.protocol = "kafka"
        proxy.name = "name"
        proxy.guid = "guid"
        proxy.type = "abcd"
        proxy.kafka_topic = "kafka_topic"
        result_proxy.message_id = "msg1"
        result_proxy.status = result
        result_proxy.proxies = []
        result_proxy.proxies.append(proxy)

        producer.produce_sync("fabric-mb-public-test2", result_proxy)

        result_pool = ResultPoolInfoAvro()
        pool = PoolInfoAvro()
        pool.name = "abc"
        pool.type = "typ1"
        pool.properties = {'test': 'value'}
        result_pool.message_id = "msg1"
        result_pool.status = result
        result_pool.pools = []
        result_pool.pools.append(pool)

        producer.produce_sync("fabric-mb-public-test2", result_pool)

        result_actor = ResultActorAvro()
        actor = ActorAvro()
        actor.name = "abcd"
        actor.owner = auth
        actor.description = "desc"
        actor.policy_module = "pol"
        actor.policy_class = "cll"
        actor.policy_guid = "guid"
        actor.actor_module = "module"
        actor.actor_class = "class"
        actor.id = "a1"
        result_actor.message_id = "msg1"
        result_actor.status = result
        result_actor.actors = []
        result_actor.actors.append(actor)

        producer.produce_sync("fabric-mb-public-test2", result_actor)


        # Fallback to earliest to ensure all messages are consumed
        conf['auto.offset.reset'] = "earliest"

        print("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++")
        print("++++++++++++++++++++++CONSUMER+++++++++++++++++++++")
        print("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++")

        class TestConsumer(AvroConsumerApi):
            def set_parent(self, parent):
                self.parent = parent

            def handle_message(self, message: IMessageAvro):
                if message.get_message_name() == IMessageAvro.Claim:
                   self.parent.validate_claim(message, claim)

                elif message.get_message_name() == IMessageAvro.ClaimDelegation:
                   self.parent.validate_claim_delegation(message, claimd)

                elif message.get_message_name() == IMessageAvro.Close:
                    self.parent.validate_close(message, close)

                elif message.get_message_name() == IMessageAvro.ExtendLease:
                    self.parent.validate_extend_lease(message, extend_lease)

                elif message.get_message_name() == IMessageAvro.ExtendTicket:
                    self.parent.validate_extend_ticket(message, extend_ticket)

                elif message.get_message_name() == IMessageAvro.FailedRPC:
                   self.parent.validate_failed_rpc(message, failed_rpc)

                elif message.get_message_name() == IMessageAvro.ModifyLease:
                    self.parent.validate_modify_lease(message, modify_lease)

                elif message.get_message_name() == IMessageAvro.Query:
                   self.parent.validate_query(message, query)

                elif message.get_message_name() == IMessageAvro.QueryResult:
                   self.parent.validate_query_result(message, query_result)

                elif message.get_message_name() == IMessageAvro.Redeem:
                   self.parent.validate_redeem(message, redeem)

                elif message.get_message_name() == IMessageAvro.UpdateLease:
                   self.parent.validate_update_lease(message, update_lease)

                elif message.get_message_name() == IMessageAvro.UpdateTicket:
                   self.parent.validate_update_ticket(message, update_ticket)

                elif message.get_message_name() == IMessageAvro.Ticket:
                   self.parent.validate_ticket(message, ticket)

                elif message.get_message_name() == IMessageAvro.ClaimResources:
                   self.parent.validate_claim_resources(message, claim_req)

                elif message.get_message_name() == IMessageAvro.AddSlice:
                    self.parent.validate_add_slice(message, add_slice)

                elif message.get_message_name() == IMessageAvro.UpdateSlice:
                    self.parent.validate_update_slice(message, update_slice)

                elif message.get_message_name() == IMessageAvro.RemoveSlice:
                    self.parent.validate_remove_slice(message, remove_slice)

                elif message.get_message_name() == IMessageAvro.RemoveReservation:
                    self.parent.validate_remove_reservation(message, remove_res)

                elif message.get_message_name() == IMessageAvro.CloseReservations:
                    self.parent.validate_close_reservation(message, close_res)

                elif message.get_message_name() == IMessageAvro.UpdateReservation:
                    self.parent.validate_update_reservation(message, update_res)

                elif message.get_message_name() == IMessageAvro.AddReservation:
                    self.parent.validate_add_reservation(message, add_res)

                elif message.get_message_name() == IMessageAvro.AddReservations:
                    self.parent.validate_add_reservations(message, add_ress)

                elif message.get_message_name() == IMessageAvro.DemandReservation:
                    self.parent.validate_demand_reservation(message, demand_res)

                elif message.get_message_name() == IMessageAvro.ExtendReservation:
                    self.parent.validate_extend_reservation(message, extend_res)

                elif message.get_message_name() == IMessageAvro.GetSlicesRequest:
                   self.parent.validate_get_slices_request(message, get_slice)

                elif message.get_message_name() == IMessageAvro.GetReservationsRequest:
                   self.parent.validate_get_reservations_request(message, res_req)

                elif message.get_message_name() == IMessageAvro.GetReservationsStateRequest:
                   self.parent.validate_get_reservations_state_request(message, res_state_req)

                elif message.get_message_name() == IMessageAvro.GetUnitRequest:
                   self.parent.validate_get_unit_request(message, ruu)

                elif message.get_message_name() == IMessageAvro.GetReservationUnitsRequest:
                   self.parent.validate_get_reservations_unit_request(message, ru)

                elif message.get_message_name() == IMessageAvro.GetPoolInfoRequest:
                   self.parent.validate_get_pool_info_request(message, pool_info)

                elif message.get_message_name() == IMessageAvro.GetActorsRequest:
                   self.parent.validate_get_actors_request(message, actors_req)

                elif message.get_message_name() == IMessageAvro.ResultSlice:
                   self.parent.validate_get_slices_response(message, slice_res)

                elif message.get_message_name() == IMessageAvro.ResultReservation:
                   self.parent.validate_get_reservations_response(message, res_res)

                elif message.get_message_name() == IMessageAvro.ResultString:
                    self.parent.validate_status_response(message, status_resp)

                elif message.get_message_name() == IMessageAvro.ResultStrings:
                    self.parent.validate_result_strings(message, res_strings)

                elif message.get_message_name() == IMessageAvro.ResultReservationState:
                    self.parent.validate_result_reservation_state(message, res_state)

                elif message.get_message_name() == IMessageAvro.ResultUnits:
                    self.parent.validate_result_units(message, result_unit)

                elif message.get_message_name() == IMessageAvro.ResultProxy:
                    self.parent.validate_result_proxy(message, result_proxy)

                elif message.get_message_name() == IMessageAvro.ResultPool:
                    self.parent.validate_result_pool(message, result_pool)

                elif message.get_message_name() == IMessageAvro.ResultActor:
                    self.parent.validate_result_actor(message, result_actor)

        # create a consumer
        consumer = TestConsumer(conf, key_schema, val_schema, topics)
        consumer.set_parent(self)

        # start a thread to consume messages
        consume_thread = Thread(target=consumer.consume_auto, daemon=True)
        # consume_thread = Thread(target = consumer.consume_sync, daemon=True)

        consume_thread.start()
        time.sleep(10)

        # trigger shutdown
        consumer.shutdown()

        # delete topics
        api.delete_topics(topics)

    def validate_query(self, incoming: QueryAvro, outgoing: QueryAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.properties, outgoing.properties)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_query_result(self, incoming: QueryResultAvro, outgoing: QueryResultAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.request_id, outgoing.request_id)
        self.assertEqual(incoming.properties, outgoing.properties)
        self.assertEqual(incoming.auth, outgoing.auth)

    def validate_failed_rpc(self, incoming: FailedRPCAvro, outgoing: FailedRPCAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.error_details, outgoing.error_details)
        self.assertEqual(incoming.request_type, outgoing.request_type)
        self.assertEqual(incoming.reservation_id, outgoing.reservation_id)
        self.assertEqual(incoming.request_id, outgoing.request_id)
        self.assertEqual(incoming.auth, outgoing.auth)

    def validate_claim_resources(self, incoming: ClaimResourcesAvro, outgoing: ClaimResourcesAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.broker_id, outgoing.broker_id)
        self.assertEqual(incoming.reservation_id, outgoing.reservation_id)
        self.assertEqual(incoming.delegation_id, outgoing.delegation_id)
        self.assertEqual(incoming.slice_id, outgoing.slice_id)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_claim(self, incoming: ClaimAvro, outgoing: ClaimAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation, outgoing.reservation)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_claim_delegation(self, incoming: ClaimDelegationAvro, outgoing: ClaimDelegationAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.delegation, outgoing.delegation)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_close(self, incoming: CloseAvro, outgoing: CloseAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation, outgoing.reservation)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_extend_lease(self, incoming: ExtendLeaseAvro, outgoing: ExtendLeaseAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation, outgoing.reservation)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_extend_ticket(self, incoming: ExtendTicketAvro, outgoing: ExtendTicketAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation, outgoing.reservation)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_modify_lease(self, incoming: ModifyLeaseAvro, outgoing: ModifyLeaseAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation, outgoing.reservation)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_update_lease(self, incoming: UpdateLeaseAvro, outgoing: UpdateLeaseAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation, outgoing.reservation)
        self.assertEqual(incoming.update_data, outgoing.update_data)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_redeem(self, incoming: RedeemAvro, outgoing: RedeemAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation, outgoing.reservation)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_update_ticket(self, incoming: UpdateTicketAvro, outgoing: UpdateTicketAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation, outgoing.reservation)
        self.assertEqual(incoming.update_data, outgoing.update_data)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_ticket(self, incoming: TicketAvro, outgoing: TicketAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation, outgoing.reservation)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_get_slices_request(self, incoming: GetSlicesRequestAvro, outgoing: GetSlicesRequestAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.slice_id, outgoing.slice_id)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_get_slices_response(self, incoming: ResultSliceAvro, outgoing: ResultSliceAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.slices, outgoing.slices)
        self.assertEqual(incoming.status, outgoing.status)

    def validate_get_reservations_request(self, incoming: GetReservationsRequestAvro, outgoing: GetReservationsRequestAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.slice_id, outgoing.slice_id)
        self.assertEqual(incoming.reservation_id, outgoing.reservation_id)
        self.assertEqual(incoming.reservation_state, outgoing.reservation_state)
        self.assertEqual(incoming.type, outgoing.type)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_get_reservations_state_request(self, incoming: GetReservationsStateRequestAvro, outgoing: GetReservationsStateRequestAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation_ids, outgoing.reservation_ids)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_get_reservations_response(self, incoming: ResultReservationAvro, outgoing: ResultReservationAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservations, outgoing.reservations)
        self.assertEqual(incoming.status, outgoing.status)

    def validate_remove_slice(self, incoming: RemoveSliceAvro, outgoing: RemoveSliceAvro):
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.slice_id, outgoing.slice_id)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_status_response(self, incoming: ResultStringAvro, outgoing: ResultStringAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.result_str, outgoing.result_str)
        self.assertEqual(incoming.status, outgoing.status)

    def validate_result_strings(self, incoming: ResultStringsAvro, outgoing: ResultStringsAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.result, outgoing.result)
        self.assertEqual(incoming.status, outgoing.status)

    def validate_result_reservation_state(self, incoming: ResultReservationStateAvro, outgoing: ResultReservationStateAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.status, outgoing.status)
        self.assertEqual(len(incoming.reservation_states), len(outgoing.reservation_states))
        for i in range(len(incoming.reservation_states)):
            self.assertEqual(incoming.reservation_states[i], outgoing.reservation_states[i])

    def validate_add_slice(self, incoming: AddSliceAvro, outgoing: AddSliceAvro):
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.slice_obj, outgoing.slice_obj)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_update_slice(self, incoming: UpdateSliceAvro, outgoing: UpdateSliceAvro):
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.slice_obj, outgoing.slice_obj)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_remove_reservation(self, incoming: RemoveReservationAvro, outgoing: RemoveReservationAvro):
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.reservation_id, outgoing.reservation_id)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_close_reservation(self, incoming: CloseReservationsAvro, outgoing: CloseReservationsAvro):
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.slice_id, outgoing.slice_id)
        self.assertEqual(incoming.reservation_id, outgoing.reservation_id)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_update_reservation(self, incoming: UpdateReservationAvro, outgoing: UpdateReservationAvro):
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.reservation_obj, outgoing.reservation_obj)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_add_reservation(self, incoming: AddReservationAvro, outgoing: AddReservationAvro):
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.reservation_obj, outgoing.reservation_obj)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_add_reservations(self, incoming: AddReservationsAvro, outgoing: AddReservationsAvro):
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)
        self.assertEqual(len(incoming.reservation_list), len(outgoing.reservation_list))
        for i in range(len(incoming.reservation_list)):
            self.assertEqual(incoming.reservation_list[i], outgoing.reservation_list[i])

    def validate_demand_reservation(self, incoming: DemandReservationAvro, outgoing: DemandReservationAvro):
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.reservation_obj, outgoing.reservation_obj)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_extend_reservation(self, incoming: ExtendReservationAvro, outgoing: ExtendReservationAvro):
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.reservation_id, outgoing.reservation_id)
        self.assertEqual(incoming.end_time, outgoing.end_time)
        self.assertEqual(incoming.new_resource_type, outgoing.new_resource_type)
        self.assertEqual(incoming.new_units, outgoing.new_units)
        self.assertEqual(incoming.request_properties, outgoing.request_properties)
        self.assertEqual(incoming.config_properties, outgoing.config_properties)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_result_units(self, incoming: ResultUnitAvro, outgoing: ResultUnitAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.status, outgoing.status)
        self.assertEqual(len(incoming.units), len(outgoing.units))
        for i in range(len(incoming.units)):
            self.assertEqual(incoming.units[i], outgoing.units[i])

    def validate_result_proxy(self, incoming: ResultProxyAvro, outgoing: ResultProxyAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.status, outgoing.status)
        self.assertEqual(len(incoming.proxies), len(outgoing.proxies))
        for i in range(len(incoming.proxies)):
            self.assertEqual(incoming.proxies[i], outgoing.proxies[i])

    def validate_result_pool(self, incoming: ResultPoolInfoAvro, outgoing: ResultPoolInfoAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.status, outgoing.status)
        self.assertEqual(len(incoming.pools), len(outgoing.pools))
        for i in range(len(incoming.pools)):
            self.assertEqual(incoming.pools[i], outgoing.pools[i])

    def validate_result_actor(self, incoming: ResultActorAvro, outgoing: ResultActorAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.status, outgoing.status)
        self.assertEqual(len(incoming.actors), len(outgoing.actors))
        for i in range(len(incoming.actors)):
            self.assertEqual(incoming.actors[i], outgoing.actors[i])

    def validate_get_unit_request(self, incoming: GetUnitAvro, outgoing: GetUnitAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.unit_id, outgoing.unit_id)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_get_reservations_unit_request(self, incoming: GetReservationUnitsAvro, outgoing: GetReservationUnitsAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.reservation_id, outgoing.reservation_id)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_get_pool_info_request(self, incoming: GetPoolInfoAvro, outgoing: GetPoolInfoAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.broker_id, outgoing.broker_id)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)

    def validate_get_actors_request(self, incoming: GetActorsAvro, outgoing: GetActorsAvro):
        self.assertEqual(incoming.name, outgoing.name)
        self.assertEqual(incoming.guid, outgoing.guid)
        self.assertEqual(incoming.message_id, outgoing.message_id)
        self.assertEqual(incoming.type, outgoing.type)
        self.assertEqual(incoming.auth, outgoing.auth)
        self.assertEqual(incoming.callback_topic, outgoing.callback_topic)


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)


class AuthAvro:
    # Use __slots__ to explicitly declare all data members.
    __slots__ = ["name", "guid", "id_token"]

    def __init__(self):
        self.name = None
        self.guid = None
        self.id_token = None

    def from_dict(self, value: dict):
        self.name = value['name']
        if 'guid' in value and value['guid'] != "null":
            self.guid = value['guid']
        if 'id_token' in value and value['id_token'] != "null":
            self.id_token = value['id_token']

    def to_dict(self) -> dict:
        """
            The Avro Python library does not support code generation.
            For this reason we must provide a dict representation of our class for serialization.
        """
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {
            "name": self.name
        }
        if self.guid is not None:
            result['guid'] = self.guid

        if self.id_token is not None:
            result['id_token'] = self.id_token
        return result

    def __str__(self):
        return "name: {} guid: {} id_token: {}".format(self.name, self.guid, self.id_token)

    def __eq__(self, other):
        if not isinstance(other, AuthAvro):
            return False
        return self.name == other.name and self.guid == other.guid and self.id_token == other.id_token

    def validate(self) -> bool:
        if self.name is None:
            return False
        return True


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric.message_bus.messages.slice_avro import SliceAvro


class DelegationAvro:
    # Use __slots__ to explicitly declare all data members.
    __slots__ = ["delegation_id", "slice", "graph", "sequence"]

    def __init__(self):
        self.delegation_id = None
        self.slice = None
        self.graph = None
        self.sequence = 0

    def from_dict(self, value: dict):
        self.delegation_id = value['delegation_id']
        self.sequence = value.get('sequence', None)
        self.slice = SliceAvro()
        self.slice.from_dict(value['slice'])
        self.graph = value.get('graph', None)

    def to_dict(self) -> dict:
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {
            "delegation_id": self.delegation_id,
            "slice": self.slice.to_dict(),
            "sequence": self.sequence
        }
        if self.graph is not None:
            result['graph'] = self.graph
        return result

    def __str__(self):
        return "delegation_id: {} slice: {} sequence: {} ".format(self.delegation_id, self.slice, self.sequence)

    def __eq__(self, other):
        if not isinstance(other, DelegationAvro):
            return False
        return self.delegation_id == other.delegation_id and self.slice == other.slice and \
               self.sequence == other.sequence and self.graph == other.graph

    def validate(self) -> bool:
        ret_val = True
        if self.delegation_id is None or self.slice is None or self.sequence is None:
            ret_val = False
        return ret_val

    def get_delegation_id(self) -> str:
        return self.delegation_id

    def get_slice_object(self) -> SliceAvro:
        return self.slice

    def get_graph(self) -> str:
        return self.graph

    def get_sequence(self) -> int:
        return self.sequence

    def print(self):
        print("")
        print("Delegation ID: {} Slice ID: {}".format(self.delegation_id, self.slice.get_slice_id()))
        if self.sequence is not None:
            print("Sequence: {}".format(self.sequence))
        if self.graph is not None :
            print("Graph: {}".format(self.graph))
        print("")


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import time

from confluent_kafka.admin import AdminClient, NewTopic, NewPartitions, ConfigResource, ConfigSource
from confluent_kafka import KafkaException

from fabric.message_bus.base import Base


class AdminApi(Base):
    def __init__(self, conf, logger=None):
        super().__init__(logger)
        self.admin_client = AdminClient(conf)

    def create_topics(self, topics, num_partitions: int = 3, replication_factor: int = 1):
        """
            Create a list of topics
            :param topics: list of topics to be created
            :param num_partitions: number of partitions for the topic
            :param replication_factor: replication factor
            :return:
        """

        new_topics = [NewTopic(topic, num_partitions=num_partitions, replication_factor=replication_factor) for topic in topics]

        # Call create_topics to asynchronously create topics, a dict
        # of <topic,future> is returned.
        fs = self.admin_client.create_topics(new_topics)

        # Wait for operation to finish.
        # Timeouts are preferably controlled by passing request_timeout=15.0
        # to the create_topics() call.
        # All futures will finish at the same time.
        for topic, f in fs.items():
            try:
                f.result()  # The result itself is None
                self.log_debug("Topic {} created".format(topic))
            except Exception as e:
                self.log_error("Failed to create topic {}: {}".format(topic, e))

    def delete_topics(self, topics):
        """
            delete list of topics
            :param topics: list of topics
            :return:
        """

        # Call delete_topics to asynchronously delete topics, a future is returned.
        # By default this operation on the broker returns immediately while
        # topics are deleted in the background. But here we give it some time (30s)
        # to propagate in the cluster before returning.
        #
        # Returns a dict of <topic,future>.
        fs = self.admin_client.delete_topics(topics, operation_timeout=30)

        # Wait for operation to finish.
        for topic, f in fs.items():
            try:
                f.result()  # The result itself is None
                self.log_debug("Topic {} deleted".format(topic))
            except Exception as e:
                self.log_error("Failed to delete topic {}: {}".format(topic, e))

    def create_partitions(self, topic_partitions):
        """
            create partitions for list of topics
            :param topic_partitions: list of tuples (topic, number of partitions)
            :return:
        """

        new_parts = [NewPartitions(topic, int(new_total_count)) for
                     topic, new_total_count in topic_partitions]

        # Try switching validate_only to True to only validate the operation
        # on the broker but not actually perform it.
        fs = self.admin_client.create_partitions(new_parts, validate_only=False)

        # Wait for operation to finish.
        for topic, f in fs.items():
            try:
                f.result()  # The result itself is None
                self.log_debug("Additional partitions created for topic {}".format(topic))
            except Exception as e:
                self.log_error("Failed to add partitions to topic {}: {}".format(topic, e))

    def _print_config(self, config, depth):
        """
            Prints the config
            :param config: config to be printed
            :param depth: number of spaces to be printed before config
            :return:
        """
        self.log_info('%40s = %-50s  [%s,is:read-only=%r,default=%r,sensitive=%r,synonym=%r,synonyms=%s]' %
                  ((' ' * depth) + config.name, config.value, ConfigSource(config.source),
                  config.is_read_only, config.is_default,
                  config.is_sensitive, config.is_synonym,
                  ["%s:%s" % (x.name, ConfigSource(x.source))
                  for x in iter(config.synonyms.values())]))

    def describe_configs(self, resources):
        """
            describe configs
            :param resources: list of tuples (resource type, resource name)
            :return:
        """

        resources = [ConfigResource(restype, resname) for
                     restype, resname in resources]

        fs = self.admin_client.describe_configs(resources)

        # Wait for operation to finish.
        for res, f in fs.items():
            try:
                configs = f.result()
                for config in iter(configs.values()):
                    self._print_config(config, 1)

            except KafkaException as e:
                self.log_error("Failed to describe {}: {}".format(res, e))
            except Exception:
                raise

    def alter_configs(self, resource_list):
        """
            Alter configs atomically, replacing non-specified
            configuration properties with their default values.
            :param resource_list: list of tuples (resource type, resource name, list of config params <config=val,config2=val2>)
            :return:
        """

        resources = []
        for restype, resname, configs in resource_list:
            resource = ConfigResource(restype, resname)
            resources.append(resource)
            for k, v in [conf.split('=') for conf in configs.split(',')]:
                resource.set_config(k, v)

        fs = self.admin_client.alter_configs(resources)

        # Wait for operation to finish.
        for res, f in fs.items():
            try:
                f.result()  # empty, but raises exception on failure
                self.log_debug("{} configuration successfully altered".format(res))
            except Exception:
                raise

    def list(self, type=None) -> list:
        """
            list topics and cluster metadata
            :param type: list topics or brokers or all; allowed values (all|topics|brokers)
            :return:
        """

        if type is None:
            what = "all"
        else:
            what = type

        md = self.admin_client.list_topics(timeout=10)

        self.log_debug("Cluster {} metadata (response from broker {}):".format(md.cluster_id, md.orig_broker_name))

        if what in ("all", "brokers"):
            self.log_debug(" {} brokers:".format(len(md.brokers)))
            for b in iter(md.brokers.values()):
                if b.id == md.controller_id:
                    self.log_debug("  {}  (controller)".format(b))
                else:
                    self.log_debug("  {}".format(b))

        if what not in ("all", "topics"):
            return

        self.log_debug(" {} topics:".format(len(md.topics)))
        for t in iter(md.topics.values()):
            if t.error is not None:
                errstr = ": {}".format(t.error)
            else:
                errstr = ""

            self.log_debug("  \"{}\" with {} partition(s){}".format(t, len(t.partitions), errstr))

            for p in iter(t.partitions.values()):
                if p.error is not None:
                    errstr = ": {}".format(p.error)
                else:
                    errstr = ""

                self.log_debug("    partition {} leader: {}, replicas: {}, isrs: {}, errstr: {}".format(
                    p.id, p.leader, p.replicas, p.isrs, errstr))

    def list_topics(self, timeout: int = 10) -> list:
        """
            list topics and cluster metadata
            :param type: list topics or brokers or all; allowed values (all|topics|brokers)
            :param timeout: timeout in ms
            :return:
        """
        result = []
        md = self.admin_client.list_topics(timeout=timeout)

        for t in iter(md.topics.values()):
            result.append(str(t))

        return result

if __name__ == '__main__':

    # create admin client
    api = AdminApi("localhost:9092")
    topics = ['topic1', "topic2"]

    # create topics
    api.create_topics(topics)
    topic_partitions = [('topic1', 4), ("topic2", 4)]

    # create partitions
    api.create_partitions(topic_partitions)
    time.sleep(5)

    # delete partitions
    api.delete_topics(topics)


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric.message_bus.messages.resource_data_avro import ResourceDataAvro


class ResourceSetAvro:
    # Use __slots__ to explicitly declare all data members.
    __slots__ = ["units", "type", "resource_data", "concrete"]

    def __init__(self):
        self.units = None
        self.type = None
        self.resource_data = None
        self.concrete = None

    def from_dict(self, value: dict):
        self.units = value['units']
        self.type = value['type']
        if value.get('resource_data', None) is not None:
            self.resource_data = ResourceDataAvro()
            self.resource_data.from_dict(value['resource_data'])
        if value.get('concrete', None) is not None:
            self.concrete = value['concrete']

    def to_dict(self) -> dict:
        """
            The Avro Python library does not support code generation.
            For this reason we must provide a dict representation of our class for serialization.
        """
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {
            "units": self.units,
            "type": self.type
        }
        if self.resource_data is not None:
            result['resource_data'] = self.resource_data.to_dict()
        if self.concrete is not None:
            result['concrete'] = self.concrete
        return result

    def __str__(self):
        return "units: {} type: {} resource_data: {} concrete: {}"\
            .format(self.units, self.type, self.resource_data, self.concrete)

    def __eq__(self, other):
        if not isinstance(other, ResourceSetAvro):
            return False

        return self.units == other.units and self.type == other.type and \
               self.resource_data == other.resource_data and self.concrete == other.concrete

    def validate(self) -> bool:
        ret_val = True
        if self.units is None or self.type is None:
            ret_val = False
        return ret_val


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)


class IMessageAvro:
    """
        Implements the base class for storing the deserialized Avro record. Each message is uniquely identified by
        a globally unique identifier. It must be inherited to include Actor specific fields and to_dict implementation.
        New Avro schema must be defined as per the inherited class and should be used for the producer/consumer
    """
    Claim = "Claim"
    ClaimDelegation = "ClaimDelegation"
    Reclaim = "Reclaim"
    ReclaimDelegation = "ReclaimDelegation"
    Close = "Close"
    ExtendLease = "ExtendLease"
    ExtendTicket = "ExtendTicket"
    FailedRPC = "FailedRPC"
    ModifyLease = "ModifyLease"
    Query = "Query"
    QueryResult = "QueryResult"
    Redeem = "Redeem"
    Relinquish = "Relinquish"
    UpdateLease = "UpdateLease"
    UpdateTicket = "UpdateTicket"
    Ticket = "Ticket"
    UpdateDelegation = "UpdateDelegation"

    # Management APIs
    ClaimResources = "ClaimResources"
    ReclaimResources = "ReclaimResources"
    RemoveSlice = "RemoveSlice"
    AddSlice = "AddSlice"
    UpdateSlice = "UpdateSlice"
    RemoveReservation = "RemoveReservation"
    CloseReservations = "CloseReservations"
    UpdateReservation = "UpdateReservation"
    AddReservation = "AddReservation"
    AddReservations = "AddReservations"
    DemandReservation = "DemandReservation"
    ExtendReservation = "ExtendReservation"

    GetReservationsStateRequest = "GetReservationsStateRequest"
    GetSlicesRequest = "GetSlicesRequest"
    GetReservationsRequest = "GetReservationsRequest"
    GetDelegations = "GetDelegations"
    GetReservationUnitsRequest = "GetReservationUnitsRequest"
    GetUnitRequest = "GetUnitRequest"
    GetPoolInfoRequest = "GetPoolInfoRequest"
    GetActorsRequest = "GetActorsRequest"

    ResultSlice = "ResultSlice"
    ResultReservation = "ResultReservation"
    ResultDelegation = "ResultDelegation"
    ResultReservationState = "ResultReservationState"
    ResultStrings = "ResultStrings"
    ResultString = "ResultString"
    ResultUnits = "ResultUnits"
    ResultProxy = "ResultProxy"
    ResultPool = "ResultPool"
    ResultActor = "ResultActor"


    def to_dict(self) -> dict:
        """
            The Avro Python library does not support code generation.
            For this reason we must provide a dict representation of our class for serialization.
        """
        raise NotImplementedError

    def from_dict(self, value: dict):
        raise NotImplementedError

    def get_message_id(self) -> str:
        raise NotImplementedError

    def get_message_name(self) -> str:
        raise NotImplementedError

    def get_callback_topic(self) -> str:
        raise NotImplementedError

    def get_id(self) -> str:
        raise NotImplementedError

    def validate(self) -> bool:
        ret_val = True
        if self.get_message_name() is None or self.get_message_id() is None or self.get_id() is None:
            ret_val = False
        return ret_val


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)


class ProxyAvro:
    # Use __slots__ to explicitly declare all data members.
    __slots__ = ["protocol", "name", "guid", "type", "kafka_topic"]

    def __init__(self):
        self.protocol = None
        self.name = None
        self.guid = None
        self.type = None
        self.kafka_topic = None

    def from_dict(self, value: dict):
        self.protocol = value.get('protocol', None)
        self.name = value.get('name', None)
        self.guid = value.get('guid', None)
        self.type = value.get('type', None)
        self.kafka_topic = value.get('kafka_topic', None)

    def to_dict(self) -> dict:
        """
            The Avro Python library does not support code generation.
            For this reason we must provide a dict representation of our class for serialization.
        """
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {
            "protocol": self.protocol,
            "name": self.name,
            "guid": self.guid,
            "type": self.type,
            "kafka_topic": self.kafka_topic
        }
        return result

    def set_name(self, name: str):
        self.name = name

    def set_guid(self, guid: str):
        self.guid = guid

    def get_name(self) -> str:
        return self.name

    def get_guid(self) -> str:
        return self.guid

    def set_protocol(self, protocol: str):
        self.protocol = protocol

    def set_type(self, type: str):
        self.type = type

    def get_protocol(self) -> str:
        return self.protocol

    def get_type(self) -> str:
        return self.type

    def set_kafka_topic(self, kafka_topic: str):
        self.kafka_topic = kafka_topic

    def get_kafka_topic(self) -> str:
        return self.kafka_topic

    def __eq__(self, other):
        if not isinstance(other, ProxyAvro):
            return False

        return self.name == other.name and self.type == other.type and self.guid == other.guid and \
               self.kafka_topic == other.kafka_topic and self.protocol == other.protocol

    def validate(self) -> bool:
        ret_val = True
        if self.name is None or self.type is None or self.guid is None or self.protocol is None:
            ret_val = False
        return ret_val


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)


class ReservationMng:
    def __init__(self):
        self.name = self.__class__.__name__
        self.reservation_id = None
        self.slice_id = None
        self.start = None
        self.end = None
        self.requested_end = None
        self.rtype = None
        self.units = None
        self.state = None
        self.pending_state = None
        self.local = None
        self.config = None
        self.request = None
        self.resource = None
        self.notices = None

    def from_dict(self, value: dict):
        self.name = value.get('name', None)
        self.reservation_id = value.get('reservation_id', None)
        self.slice_id = value.get('slice_id', None)
        self.start = value.get('start', None)
        self.end = value.get('end', None)
        self.requested_end = value.get('requested_end', None)
        self.rtype = value.get('rtype', None)
        self.units = value.get('units', None)
        self.state = value.get('state', None)
        self.pending_state = value.get('pending_state', None)
        self.local = value.get('local', None)
        self.config = value.get('config', None)
        self.request = value.get('request', None)
        self.resource = value.get('resource', None)
        self.notices = value.get('notices', None)

    def to_dict(self) -> dict:
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {'name': self.name,
                  'reservation_id': self.reservation_id,
                  'rtype': self.rtype,
                  'notices': self.notices}
        if self.slice_id is not None:
            result['slice_id'] = self.slice_id

        if self.start is not None:
            result['start'] = self.start

        if self.end is not None:
            result['end'] = self.end

        if self.requested_end is not None:
            result['requested_end'] = self.requested_end

        if self.units is not None:
            result['units'] = self.units

        if self.state is not None:
            result['state'] = self.state

        if self.pending_state is not None:
            result['pending_state'] = self.pending_state

        if self.local is not None:
            result['local'] = self.local

        if self.config is not None:
            result['config'] = self.config

        if self.request is not None:
            result['request'] = self.request

        if self.resource is not None:
            result['resource'] = self.resource

        return result

    def print(self):
        print("")
        print("Reservation ID: {} Slice ID: {}".format(self.reservation_id, self.slice_id))
        if self.rtype is not None or self.notices is not None:
            print("Resource Type: {} Notices: {}".format(self.rtype, self.notices))

        if self.start is not None or self.end is not None or self.requested_end is not None:
            print("Start: {} End: {} Requested End: {}".format(self.start, self.end, self.requested_end))

        if self.units is not None or self.state is not None or self.pending_state is not None:
            print("Units: {} State: {} Pending State: {}".format(self.units, self.state, self.pending_state))

        if self.local is not None:
            print("Local Properties: {}".format(self.local))
        if self.config is not None:
            print("Config Properties: {}".format(self.config))
        if self.request is not None:
            print("Request Properties: {}".format(self.request))
        if self.resource is not None:
            print("Resource Properties: {}".format(self.resource))
        print("")

    def __str__(self):
        return "name: {} reservation_id: {} slice_id: {} start: {} end: {} requested_end: {} rtype: {} units: {} " \
               "state: {} pending_state: {} local: {} config: {} request: {} resource: {} notices: {}"\
            .format(self.name, self.reservation_id, self.slice_id, self.start, self.end, self.requested_end,
                    self.rtype, self.units, self.state, self.pending_state, self.local, self.config, self.request,
                    self.resource, self.notices)

    def get_reservation_id(self) -> str:
        return self.reservation_id

    def set_reservation_id(self, value: str):
        self.reservation_id = value

    def get_slice_id(self) -> str:
        return self.slice_id

    def set_slice_id(self, value: str):
        self.slice_id = value

    def get_start(self) -> int:
        return self.start

    def set_start(self, value: int):
        self.start = value

    def get_end(self) -> int:
        return self.end

    def set_end(self, value: int):
        self.end = value

    def get_requested_end(self) -> int:
        return self.requested_end

    def set_requested_end(self, value: int):
        self.requested_end = value

    def get_resource_type(self) -> str:
        return self.rtype

    def set_resource_type(self, value: str):
        self.rtype = value

    def get_units(self) -> int:
        return self.units

    def set_units(self, value: int):
        self.units = value

    def get_state(self) -> int:
        return self.state

    def set_state(self, value: int):
        self.state = value

    def get_pending_state(self) -> int:
        return self.pending_state

    def set_pending_state(self, value: int):
        self.pending_state = value

    def get_local_properties(self) -> dict:
        return self.local

    def set_local_properties(self, value: dict):
        self.local = value

    def get_config_properties(self) -> dict:
        return self.config

    def set_config_properties(self, value: dict):
        self.config = value

    def get_request_properties(self) -> dict:
        return self.request

    def set_request_properties(self, value: dict):
        self.request = value

    def get_resource_properties(self) -> dict:
        return self.resource

    def set_resource_properties(self, value: dict):
        self.resource = value

    def get_notices(self) -> str:
        return self.notices

    def set_notices(self, value: str):
        self.notices = value

    def __eq__(self, other):
        if not isinstance(other, ReservationMng):
            return False

        return self.name == other.name and self.reservation_id == other.reservation_id and \
            self.slice_id == other.slice_id and self.start == other.start and self.end == other.end and \
            self.requested_end == other.requested_end and self.rtype == other.rtype and self.units == other.units and \
            self.state == other.state and self.pending_state == other.pending_state and self.local == other.local and \
            self.request == other.request and self.resource == other.resource and self.notices == other.notices

    def validate(self) -> bool:
        ret_val = True
        if self.reservation_id is None or self.rtype is None or self.notices is None:
            print("reservation_id: {} rtype: {} notices: {}".format(self.reservation_id, self.rtype, self.notices))
            ret_val = False
        return ret_val


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)

from __future__ import annotations

from fabric.message_bus.messages.reservation_predecessor_avro import ReservationPredecessorAvro
from fabric.message_bus.messages.ticket_reservation_avro import TicketReservationAvro


class LeaseReservationAvro(TicketReservationAvro):
    def __init__(self):
        super().__init__()
        self.authority = None
        self.join_state = None
        self.leased_units = None
        self.redeem_processors = []
        self.name = self.__class__.__name__

    def from_dict(self, value: dict):
        super().from_dict(value)
        self.authority = value.get('authority', None)
        self.join_state = value.get('join_state', None)
        self.leased_units = value.get('leased_units', None)
        temp_redeem = value.get('redeem_processors', None)
        if temp_redeem is not None:
            for p in temp_redeem:
                predecessor = ReservationPredecessorAvro()
                predecessor.from_dict(p)
                self.redeem_processors.append(predecessor)

    def to_dict(self) -> dict:
        if not self.validate():
            raise Exception("Invalid arguments")

        result = super().to_dict()
        if result is None:
            result = {}
        result['authority'] = self.authority
        if self.join_state is not None:
            result['join_state'] = self.join_state

        if self.leased_units is not None:
            result['leased_units'] = self.leased_units

        if self.redeem_processors is not None and len(self.redeem_processors) > 0:
            temp = []
            for p in self.redeem_processors:
                temp.append(p.to_dict())
            result['redeem_processors'] = temp

        return result

    def __str__(self):
        return "{} authority: {} join_state: {} leased_units: {} redeem_processors: {}".format(super().__str__(),
                                                                                               self.authority,
                                                                                               self.join_state,
                                                                                               self.leased_units,
                                                                                               self.redeem_processors)

    def print(self):
        print("")
        print("Reservation ID: {} Slice ID: {}".format(self.reservation_id, self.slice_id))
        if self.rtype is not None or self.notices is not None:
            print("Resource Type: {} Notices: {}".format(self.rtype, self.notices))

        if self.start is not None or self.end is not None or self.requested_end is not None:
            print("Start: {} End: {} Requested End: {}".format(self.start, self.end, self.requested_end))

        if self.units is not None or self.state is not None or self.pending_state is not None:
            print("Units: {} State: {} Pending State: {}".format(self.units, self.state, self.pending_state))

        print("Broker: {}".format(self.broker))

        if self.ticket is not None:
            print("Ticket properties: {}".format(self.ticket))

        if self.renewable is not None:
            print("Renewable: {}".format(self.renewable))

        if self.renew_time is not None:
            print("Renew Time: {}".format(self.renew_time))

        print("Authority: {}".format(self.authority))

        if self.join_state is not None:
            print("Join State: {}".format(self.join_state))

        if self.leased_units is not None:
            print("Leased Units: {}".format(self.leased_units))

        if self.redeem_processors is not None:
            index = 0
            for rp in self.redeem_processors:
                print("Redeem Predecessor# {}: {}".format(index, rp))
                index += 1

        if self.local is not None:
            print("Local Properties: {}".format(self.local))
        if self.config is not None:
            print("Config Properties: {}".format(self.config))
        if self.request is not None:
            print("Request Properties: {}".format(self.request))
        if self.resource is not None:
            print("Resource Properties: {}".format(self.resource))
        print("")

    def get_authority(self) -> str:
        return self.authority

    def set_authority(self, value: str):
        self.authority = value

    def get_join_state(self) -> int:
        return self.join_state

    def set_join_state(self, value: int):
        self.join_state = value

    def get_leased_units(self) -> int:
        return self.leased_units

    def set_leased_units(self, value: int):
        self.leased_units = value

    def get_redeem_predecessors(self) -> list:
        return self.redeem_processors

    def __eq__(self, other):
        if not isinstance(other, LeaseReservationAvro):
            return False

        return self.name == other.name and self.reservation_id == other.reservation_id and \
                self.slice_id == other.slice_id and self.start == other.start and self.end == other.end and \
                self.requested_end == other.requested_end and self.rtype == other.rtype and self.units == other.units and \
                self.state == other.state and self.pending_state == other.pending_state and self.local == other.local and \
                self.request == other.request and self.resource == other.resource and self.notices == other.notices and \
                self.broker == other.broker and self.ticket == other.ticket and self.renewable == other.renewable and \
                self.renewable == other.renew_time and \
                self.authority == other.authority and self.join_state == other.join_state and \
                self.leased_units == other.leased_units and self.redeem_processors == other.redeem_processors


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)


class TermAvro:
    # Use __slots__ to explicitly declare all data members.
    __slots__ = ["start_time", "end_time", "ticket_time", "new_start_time"]

    def __init__(self):
        self.start_time = None
        self.end_time = None
        self.ticket_time = None
        self.new_start_time = None

    def from_dict(self, value: dict):
        self.start_time = value.get('start_time', None)
        self.end_time = value.get('end_time', None)
        self.ticket_time = value.get('ticket_time', None)
        self.new_start_time = value.get('new_start_time', None)

    def to_dict(self) -> dict:
        """
            The Avro Python library does not support code generation.
            For this reason we must provide a dict representation of our class for serialization.
        """
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {
            "start_time": self.start_time,
            "end_time": self.end_time
        }
        if self.ticket_time is not None:
            result["ticket_time"] = self.ticket_time

        if self.new_start_time is not None:
            result["new_start_time"] = self.new_start_time
        return result

    def __str__(self):
        return "start_time: {} end_time: {} ticket_time: {} new_start_time: {}"\
            .format(self.start_time, self.end_time, self.ticket_time, self.new_start_time)

    def __eq__(self, other):
        if not isinstance(other, TermAvro):
            return False

        return self.start_time == other.start_time and self.ticket_time == other.ticket_time and \
               self.end_time == other.end_time and self.new_start_time == other.new_start_time

    def validate(self) -> bool:
        ret_val = True
        if self.start_time is None or self.end_time is None or self.new_start_time is None:

            ret_val = False
        return ret_val


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric.message_bus.messages.auth_avro import AuthAvro


class SliceAvro:
    # Use __slots__ to explicitly declare all data members.
    __slots__ = ["slice_name", "guid", "owner", "description", "local_properties", "config_properties",
                 "request_properties", "resource_properties", "resource_type", "client_slice", "broker_client_slice",
                 "graph_id"]

    def __init__(self):
        self.slice_name = None
        self.guid = None
        self.owner = None
        self.description = None
        self.local_properties = None
        self.config_properties = None
        self.request_properties = None
        self.resource_properties = None
        self.resource_type = None
        self.client_slice = False
        self.broker_client_slice = False
        self.graph_id = None

    def from_dict(self, value: dict):
        self.slice_name = value['slice_name']
        self.guid = value['guid']
        if value.get('owner', None) is not None:
            self.owner = AuthAvro()
            self.owner.from_dict(value['owner'])

        self.description = value.get('description', None)
        self.local_properties = value.get('local_properties', None)
        self.config_properties = value.get('config_properties', None)
        self.request_properties = value.get('request_properties', None)

        self.resource_properties = value.get('resource_properties', None)
        self.resource_type = value.get('resource_type', None)
        self.client_slice = value.get('client_slice', None)
        self.broker_client_slice = value.get('broker_client_slice', None)
        self.graph_id = value.get('graph_id', None)

    def to_dict(self) -> dict:
        """
            The Avro Python library does not support code generation.
            For this reason we must provide a dict representation of our class for serialization.
        """
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {
            "slice_name": self.slice_name,
            "guid": self.guid
        }
        if self.owner is not None:
            result["owner"] = self.owner.to_dict()

        if self.description is not None:
            result["description"] = self.description

        if self.local_properties is not None:
            result["local_properties"] = self.local_properties

        if self.config_properties is not None:
            result["config_properties"] = self.config_properties

        if self.request_properties is not None:
            result["request_properties"] = self.request_properties

        if self.resource_properties is not None:
            result["resource_properties"] = self.resource_properties

        if self.resource_type is not None:
            result["resource_type"] = self.resource_type

        if self.client_slice is not None:
            result["client_slice"] = self.client_slice

        if self.broker_client_slice is not None:
            result["broker_client_slice"] = self.broker_client_slice

        if self.graph_id is not None:
            result["graph_id"] = self.graph_id
        return result

    def __str__(self):
        return "slice_name: {} guid: {} owner: {} description: {} local_properties: {} config_properties: {} " \
               "request_properties: {} resource_properties: {} resource_type: {} client_slice: {} " \
               "broker_client_slice: {} graph_id: {}".format(self.slice_name, self.guid, self.owner, self.description,
                                                self.local_properties, self.config_properties,
                                                self.request_properties, self.resource_properties, self.resource_type,
                                                self.client_slice, self.broker_client_slice, self.graph_id)

    def print(self, all: bool = False):
        print("")
        print("Slice Name: {} Slice Guid: {}".format(self.slice_name, self.guid))
        if self.graph_id is not None:
            print("Graph ID: {}".format(self.graph_id))
        if all:
            if self.owner is not None:
                print("Slice owner: {}".format(self.owner))
            if self.description is not None:
                print("Description: {}".format(self.description))
            if self.resource_type is not None:
                print("resource_type: {}".format(self.resource_type))
            if self.client_slice is not None:
                print("client_slice: {}".format(self.client_slice))
            if self.broker_client_slice is not None:
                print("broker_client_slice: {}".format(self.broker_client_slice))

            if self.local_properties is not None:
                print("local_properties: {}".format(self.local_properties))
            if self.config_properties is not None:
                print("config_properties: {}".format(self.config_properties))
            if self.request_properties is not None:
                print("request_properties: {}".format(self.request_properties))
            if self.resource_properties is not None:
                print("resource_properties: {}".format(self.resource_properties))
        print("")

    def set_slice_name(self, value: str):
        self.slice_name = value

    def get_slice_name(self) -> str:
        return self.slice_name

    def set_owner(self, value: AuthAvro):
        self.owner = value

    def get_owner(self) -> AuthAvro:
        return self.owner

    def set_description(self, value: str):
        self.description = value

    def get_description(self) -> str:
        return self.description

    def get_local_properties(self) -> dict:
        return self.local_properties

    def set_local_properties(self, value: dict):
        self.local_properties = value

    def get_config_properties(self) -> dict:
        return self.config_properties

    def set_config_properties(self, value: dict):
        self.config_properties = value

    def get_request_properties(self) -> dict:
        return self.request_properties

    def set_request_properties(self, value: dict):
        self.request_properties = value

    def get_resource_properties(self) -> dict:
        return self.resource_properties

    def set_resource_properties(self, value: dict):
        self.resource_properties = value

    def get_resource_type(self) -> str:
        return self.resource_type

    def set_resource_type(self, value: str):
        self.resource_type = value

    def is_client_slice(self):
        return self.client_slice

    def set_client_slice(self, value : bool):
        self.client_slice = value

    def is_broker_client_slice(self):
        return self.broker_client_slice

    def set_broker_client_slice(self, value: bool):
        self.broker_client_slice = value

    def get_slice_id(self) -> str:
        return self.guid

    def set_slice_id(self, slice_id: str):
        self.guid = slice_id

    def __eq__(self, other):
        if not isinstance(other, SliceAvro):
            return False

        return self.slice_name == other.slice_name and self.guid == other.guid and self.owner == other.owner and \
            self.description == other.description and self.local_properties == other.local_properties and \
            self.config_properties == other.config_properties and self.request_properties == other.request_properties and \
            self.resource_properties == other.resource_properties and self.resource_type == other.resource_type and \
            self.client_slice == other.client_slice and self.broker_client_slice == other.broker_client_slice

    def validate(self) -> bool:
        ret_val = True
        if self.slice_name is None or self.owner is None or self.description is None or \
                self.broker_client_slice is None or self.guid is None:
            ret_val = False
        return ret_val

    def get_graph_id(self) -> str:
        return self.graph_id


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric.message_bus.messages.reservation_state_avro import ReservationStateAvro


class LeaseReservationStateAvro(ReservationStateAvro):
    def __init__(self):
        super().__init__()
        self.name = self.__class__.__name__
        self.joining = None

    def get_joining(self) -> int:
        return self.joining

    def set_joining(self, value: int):
        self.joining = value

    def from_dict(self, values: dict):
        self.name = values.get('name', None)
        self.state = values.get('state', None)
        self.pending_state = values.get('pending_state', None)
        self.joining = values.get('joining', None)

    def to_dict(self) -> dict:
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {
            'name':self.name,
            'state':self.state,
            'pending_state': self.pending_state}
        if self.joining is not None:
            result['joining'] = self.joining
        return result

    def __str__(self):
        return "state: {} pending_state: {} joining: {}".format(self.state, self.pending_state, self.joining)



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)


class ResultAvro:
    # Use __slots__ to explicitly declare all data members.
    __slots__ = ["code", "message", "details"]

    def __init__(self):
        self.code = 0
        self.message = ""
        self.details = ""

    def from_dict(self, value: dict):
        self.code = value.get("code", None)
        self.message = value.get("message", None)
        self.details = value.get("details", None)

    def to_dict(self) -> dict:
        """
            The Avro Python library does not support code generation.
            For this reason we must provide a dict representation of our class for serialization.
        """
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {
            "code": self.code
        }
        if self.message is not None:
            result["message"] = self.message
        if self.details is not None:
            result["details"] = self.details
        return result

    def __str__(self):
        return "code: {} message: {} details: {}".format(self.code, self.message, self.details)

    def get_code(self) -> int:
        return self.code

    def set_code(self, code: int):
        self.code = code

    def get_message(self) -> str:
        return self.message

    def set_message(self, msg: str):
        self.message = msg

    def get_details(self) -> str:
        return self.details

    def set_details(self, value: str):
        self.details = value

    def __eq__(self, other):
        if not isinstance(other, ResultAvro):
            return False

        return self.code == other.code and self.message == other.message and self.details == other.details

    def validate(self) -> bool:
        ret_val = True
        if self.code is None:
            ret_val = False
        return ret_val


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from uuid import uuid4

from fabric.message_bus.messages.auth_avro import AuthAvro
from fabric.message_bus.messages.reservation_mng import ReservationMng
from fabric.message_bus.messages.message import IMessageAvro


class DemandReservationAvro(IMessageAvro):
    # Use __slots__ to explicitly declare all data members.
    __slots__ = ["name", "message_id", "guid", "auth", "reservation_id", "reservation_obj", "callback_topic", "id"]

    def __init__(self):
        self.name = IMessageAvro.DemandReservation
        self.message_id = None
        self.guid = None
        self.auth = None
        self.reservation_id = None
        self.reservation_obj = None
        self.callback_topic = None
        # Unique id used to track produce request success/failures.
        # Do *not* include in the serialized object.
        self.id = uuid4()

    def from_dict(self, value: dict):
        if value['name'] != IMessageAvro.DemandReservation:
            raise Exception("Invalid message")
        self.message_id = value['message_id']
        self.guid = value['guid']
        self.callback_topic = value['callback_topic']
        self.reservation_id = value.get('reservation_id', None)

        if value.get("reservation_obj", None) is not None:
            res_value = value.get("reservation_obj", None)
            self.reservation_obj = ReservationMng()
            self.reservation_obj.from_dict(res_value)

        if value.get('auth', None) is not None:
            self.auth = AuthAvro()
            self.auth.from_dict(value['auth'])

    def to_dict(self) -> dict:
        """
            The Avro Python library does not support code generation.
            For this reason we must provide a dict representation of our class for serialization.
        """
        if not self.validate():
            raise Exception("Invalid arguments")
        result = {
            "name": self.name,
            "message_id": self.message_id,
            "guid": self.guid,
            "callback_topic": self.callback_topic
        }
        if self.auth is not None:
            result['auth'] = self.auth.to_dict()

        if self.reservation_obj is not None:
            result['reservation_obj'] = self.reservation_obj.to_dict()

        if self.reservation_id is not None:
            result['reservation_id'] = self.reservation_id

        return result

    def get_message_id(self) -> str:
        """
        Returns the message_id
        """
        return self.message_id

    def get_message_name(self) -> str:
        return self.name

    def get_callback_topic(self) -> str:
        return self.callback_topic

    def get_id(self) -> str:
        return self.id.__str__()

    def get_reservation(self) -> ReservationMng:
        return self.reservation_obj

    def get_reservation_id(self) -> str:
        return self.reservation_id

    def __str__(self):
        return "name: {} message_id: {} guid: {} auth: {} reservation_id: {} reservation_obj: {} " \
               "callback_topic: {}".format(self.name, self.message_id, self.guid,
                                           self.auth, self.reservation_id, self.reservation_obj, self.callback_topic)

    def validate(self) -> bool:
        ret_val = super().validate()
        if self.guid is None or self.auth is None or self.callback_topic is None or \
                (self.reservation_id is None and self.reservation_obj is None):
            ret_val = False
        return ret_val


# coding: utf-8

from setuptools import setup, find_packages

from fabric import SchemaDir, MessageSchemaFile, KeySchemaFile

NAME = "fabric-message-bus"
VERSION = "0.3"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["confluent-kafka[avro]",
            "confluent-kafka"]

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name=NAME,
    version=VERSION,
    description="Fabric Python Message Bus Library",
    author="Komal Thareja",
    author_email="kthare10@renci.org",
    url="https://github.com/fabric-testbed/MessageBus",
    keywords=["Kafka", "Fabric Message Bus"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    data_files=[(SchemaDir, [KeySchemaFile, MessageSchemaFile])],
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
                  "Programming Language :: Python :: 3",
                  "License :: OSI Approved :: MIT License",
                  "Operating System :: OS Independent",
              ],
    python_requires='>=3.7'
)



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric.message_bus.messages.auth_avro import AuthAvro


class ActorAvro:
    def __init__(self):
        self.name = None
        self.type = None
        self.owner = None
        self.description = None
        self.policy_class = None
        self.policy_module = None
        self.actor_class = None
        self.actor_module = None
        self.online = None
        self.management_class = None
        self.management_module = None
        self.id = None
        self.policy_guid = None

    def __eq__(self, other):
        if not isinstance(other, ActorAvro):
            return False

        return self.name == other.name and self.type == other.type and self.description == other.description and \
               self.policy_class == other.policy_class and self.policy_module == other.policy_module and \
               self.actor_class == other.actor_class and self.actor_module == other.actor_module and \
               self.online == other.online and self.management_module == other.management_module and \
               self.management_class == other.management_class and self.id == other.id and \
               self.policy_guid == other.policy_guid

    def from_dict(self, value: dict):
        self.name = value.get('name', None)
        self.type = value.get('type', None)
        self.owner = value.get('owner', None)
        self.description = value.get('description', None)
        self.policy_class = value.get('policy_class', None)
        self.policy_module = value.get('policy_module', None)
        self.actor_class = value.get('actor_class', None)
        self.actor_module = value.get('actor_module', None)
        self.online = value.get('online', None)
        self.management_class = value.get('management_class', None)
        self.management_module = value.get('management_module', None)
        self.id = value.get('id', None)
        self.policy_guid = value.get('policy_guid', None)

    def to_dict(self) -> dict:
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {
            "name": self.name
        }
        if self.type is not None:
            result['type'] = self.type
        if self.owner is not None:
            result['owner'] = self.owner.to_dict()
        if self.description is not None:
            result['description'] = self.description
        if self.policy_class is not None:
            result['policy_class'] = self.policy_class
        if self.policy_module is not None:
            result['policy_module'] = self.policy_module
        if self.actor_class is not None:
            result['actor_class'] = self.actor_class
        if self.actor_module is not None:
            result['actor_module'] = self.actor_module
        if self.online is not None:
            result['online'] = self.online
        if self.management_class is not None:
            result['management_class'] = self.management_class
        if self.management_module is not None:
            result['management_module'] = self.management_module
        if self.id is not None:
            result['id'] = self.id
        if self.policy_guid is not None:
            result['policy_guid'] = self.policy_guid
        return result

    def __str__(self):
        return "name: {} type: {} owner: {} description: {} policy_class: {} policy_module: {} actor_class: {}" \
               " actor_module: {} online: {} management_class: {} management_module: {} id: {} policy_guid: {}".format(
            self.name, self.type, self.owner, self.description, self.policy_class, self.policy_module, self.actor_class,
        self.actor_module, self.online, self.management_class, self.management_module, self.id, self.policy_guid)

    def get_name(self) -> str:
        return self.name

    def set_name(self, value: str):
        self.name = value

    def set_type(self, value: int):
        self.type = value

    def get_type(self) -> int:
        return self.type

    def set_owner(self, value: AuthAvro):
        self.owner = value

    def get_owner(self) -> AuthAvro:
        return self.owner

    def set_description(self, value: str):
        self.description = value

    def get_description(self) -> str:
        return self.description

    def get_policy_class(self) -> str:
        return self.policy_class

    def set_policy_class(self, value: str):
        self.policy_class = value

    def get_policy_module(self) -> str:
        return self.policy_module

    def set_policy_module(self, value: str):
        self.policy_module = value

    def get_actor_class(self) -> str:
        return self.actor_class

    def set_actor_class(self, value: str):
        self.actor_class = value

    def get_actor_module(self) -> str:
        return self.actor_module

    def set_actor_module(self, value: str):
        self.actor_module = value

    def set_online(self, value: bool):
        self.online = value

    def get_online(self) -> bool:
        return self.online

    def get_management_class(self) -> str:
        return self.management_class

    def set_management_class(self, value: str):
        self.management_class = value

    def get_management_module(self) -> str:
        return self.management_module

    def set_management_module(self, value: str):
        self.management_module = value

    def set_id(self, value: str):
        self.id = value

    def get_id(self) -> str:
        return self.id

    def set_policy_guid(self, value: str):
        self.policy_guid = value

    def get_policy_guid(self) -> str:
        return self.policy_guid

    def validate(self) -> bool:
        ret_val = True
        if self.name is None or self.owner is None or self.description is None or self.policy_module is None or \
            self.policy_class is None or self.policy_guid is None or self.actor_class is None or \
                self.actor_module is None or self.id is None:
            ret_val = False
        return ret_val


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import traceback

from confluent_kafka.avro import AvroProducer

from fabric.message_bus.admin import AdminApi
from fabric.message_bus.base import Base
from fabric.message_bus.messages.query_avro import QueryAvro
from fabric.message_bus.messages.query_result_avro import QueryResultAvro
from fabric.message_bus.messages.message import IMessageAvro


class AvroProducerApi(Base):
    """
        This class implements the Interface for Kafka producer carrying Avro messages.
        It is expected that the users would extend this class and override on_delivery function.
    """
    def __init__(self, conf, key_schema, record_schema, logger=None):
        """
            Initialize the Producer API
            :param conf: configuration e.g:
                        {'bootstrap.servers': localhost:9092,
                        'schema.registry.url': http://localhost:8083}
            :param key_schema: loaded AVRO schema for the key
            :param record_schema: loaded AVRO schema for the value
        """
        super().__init__(logger)
        self.producer = AvroProducer(conf, default_key_schema=key_schema, default_value_schema=record_schema)

    def set_logger(self, logger):
        self.logger = logger

    def delivery_report(self, err, msg, obj):

        """
            Handle delivery reports served from producer.poll.
            This callback takes an extra argument, obj.
            This allows the original contents to be included for debugging purposes.
        """
        if err is not None:
            self.log_error('Message {} delivery failed for user {} with error {}'.format(
                obj.id, obj.name, err))
        else:
            self.log_debug('Message {} successfully produced to {} [{}] at offset {}'.format(
                obj.id, msg.topic(), msg.partition(), msg.offset()))

    def produce_async(self, topic, record: IMessageAvro) -> bool:
        """
            Produce records for a specific topic
            :param topic: topic to which messages are written to
            :param record: record/message to be written
            :return:
        """
        self.log_debug("Producing records to topic {}.".format(topic))
        try:
            # The message passed to the delivery callback will already be serialized.
            # To aid in debugging we provide the original object to the delivery callback.
            self.producer.produce(topic=topic, key=record.get_id(), value=record.to_dict(),
                             callback=lambda err, msg, obj=record: self.delivery_report(err, msg, obj))
            # Serve on_delivery callbacks from previous asynchronous produce()
            self.producer.poll(0)
            return True
        except ValueError as e:
            traceback.print_exc()
            self.log_error("Invalid input, discarding record...{}".format(e))
        return False

    def produce_sync(self, topic, record: IMessageAvro) -> bool:
        """
            Produce records for a specific topic
            :param topic: topic to which messages are written to
            :param record: record/message to be written
            :return:
        """
        self.log_debug("Record type={}".format(type(record)))
        self.log_debug("Producing key {} to topic {}.".format(record.get_id(), topic))
        self.log_debug("Producing record {} to topic {}.".format(record.to_dict(), topic))

        try:
            # Pass the message synchronously
            self.producer.produce(topic=topic, key=record.get_id(), value=record.to_dict())
            self.producer.flush()
            return True
        except ValueError as e:
            traceback.print_exc()
            self.log_error("Invalid input, discarding record...")
            self.log_error(str(e))
            print(e)
        return False


if __name__ == '__main__':
    from confluent_kafka import avro

    # Create Admin API object
    api = AdminApi("localhost:9092")
    topics = ['topic1']

    # create topics
    api.create_topics(topics)

    conf = {'bootstrap.servers': "localhost:9092",
            'schema.registry.url': "http://localhost:8081"}

    # load AVRO schema
    key_schema = avro.loads(open('schema/key.avsc', "r").read())
    val_schema = avro.loads(open('schema/message.avsc', "r").read())

    # create a producer
    producer = AvroProducerApi(conf, key_schema, val_schema)

    # produce message to topics
    message = QueryAvro()
    message.message_id = "msg1"
    message.properties = {"abc": "def"}
    producer.produce_sync("topic1", message)

    message = QueryResultAvro()
    message.message_id = "msg2"
    message.properties = {"abc": "def"}
    producer.produce_sync("topic1", message)

    # delete topics
    api.delete_topics(topics)


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)


from confluent_kafka.avro import AvroConsumer, SerializerError
from confluent_kafka.cimpl import KafkaError

from fabric.message_bus.base import Base
from fabric.message_bus.messages.add_reservation_avro import AddReservationAvro
from fabric.message_bus.messages.add_reservations_avro import AddReservationsAvro
from fabric.message_bus.messages.add_slice_avro import AddSliceAvro
from fabric.message_bus.messages.claim_avro import ClaimAvro
from fabric.message_bus.messages.claim_delegation_avro import ClaimDelegationAvro
from fabric.message_bus.messages.claim_resources_avro import ClaimResourcesAvro
from fabric.message_bus.messages.close_avro import CloseAvro
from fabric.message_bus.messages.close_reservations_avro import CloseReservationsAvro
from fabric.message_bus.messages.demand_reservation_avro import DemandReservationAvro
from fabric.message_bus.messages.extend_lease_avro import ExtendLeaseAvro
from fabric.message_bus.messages.extend_reservation_avro import ExtendReservationAvro
from fabric.message_bus.messages.extend_ticket_avro import ExtendTicketAvro
from fabric.message_bus.messages.failed_rpc_avro import FailedRPCAvro
from fabric.message_bus.messages.get_delegations_avro import GetDelegationsAvro
from fabric.message_bus.messages.get_pool_info_avro import GetPoolInfoAvro
from fabric.message_bus.messages.get_reservation_units_avro import GetReservationUnitsAvro
from fabric.message_bus.messages.get_reservations_request_avro import GetReservationsRequestAvro
from fabric.message_bus.messages.get_unit_avro import GetUnitAvro
from fabric.message_bus.messages.reclaim_avro import ReclaimAvro
from fabric.message_bus.messages.reclaim_delegation_avro import ReclaimDelegationAvro
from fabric.message_bus.messages.reclaim_resources_avro import ReclaimResourcesAvro
from fabric.message_bus.messages.result_actor_avro import ResultActorAvro
from fabric.message_bus.messages.result_delegation_avro import ResultDelegationAvro
from fabric.message_bus.messages.result_pool_info_avro import ResultPoolInfoAvro
from fabric.message_bus.messages.result_proxy_avro import ResultProxyAvro
from fabric.message_bus.messages.result_reservation_avro import ResultReservationAvro
from fabric.message_bus.messages.get_reservations_state_request_avro import GetReservationsStateRequestAvro
from fabric.message_bus.messages.result_reservation_state_avro import ResultReservationStateAvro
from fabric.message_bus.messages.get_slices_request_avro import GetSlicesRequestAvro
from fabric.message_bus.messages.result_slice_avro import ResultSliceAvro
from fabric.message_bus.messages.modify_lease_avro import ModifyLeaseAvro
from fabric.message_bus.messages.query_avro import QueryAvro
from fabric.message_bus.messages.query_result_avro import QueryResultAvro
from fabric.message_bus.messages.redeem_avro import RedeemAvro
from fabric.message_bus.messages.relinquish_avro import RelinquishAvro
from fabric.message_bus.messages.remove_reservation_avro import RemoveReservationAvro
from fabric.message_bus.messages.remove_slice_avro import RemoveSliceAvro
from fabric.message_bus.messages.result_string_avro import ResultStringAvro
from fabric.message_bus.messages.result_strings_avro import ResultStringsAvro
from fabric.message_bus.messages.result_unit_avro import ResultUnitAvro
from fabric.message_bus.messages.ticket_avro import TicketAvro
from fabric.message_bus.messages.update_delegation_avro import UpdateDelegationAvro
from fabric.message_bus.messages.update_lease_avro import UpdateLeaseAvro
from fabric.message_bus.messages.update_reservation_avro import UpdateReservationAvro
from fabric.message_bus.messages.update_slice_avro import UpdateSliceAvro
from fabric.message_bus.messages.update_ticket_avro import UpdateTicketAvro
from fabric.message_bus.messages.message import IMessageAvro


class AvroConsumerApi(Base):
    def __init__(self, conf, key_schema, record_schema, topics, batchSize = 5, logger=None):
        super().__init__(logger)
        self.consumer = AvroConsumer(conf, reader_key_schema=key_schema, reader_value_schema=record_schema)
        self.running = True
        self.topics = topics
        self.batch_size = batchSize

    def shutdown(self):
        """
        Shutdown the consumer
        :return:
        """
        self.log_debug("Trigger shutdown")
        self.running = False

    def process_message(self, topic: str, key: dict, value: dict):
        """
        Process the incoming message. Must be overridden in the derived class
        :param topic: topic name
        :param key: incoming message key
        :param value: incoming message value
        :return:
        """
        self.log_debug("Message received for topic " + topic)
        self.log_debug("Key = {}".format(key))
        self.log_debug("Value = {}".format(value))
        message = None
        if value['name'] == IMessageAvro.Query:
            message = QueryAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.QueryResult:
            message = QueryResultAvro()
            message.from_dict(value)
            self.log_debug("QueryResult: {}".format(message))
        elif value['name'] == IMessageAvro.FailedRPC:
            message = FailedRPCAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.Redeem:
            message = RedeemAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.Ticket:
            message = TicketAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.Claim:
            message = ClaimAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.Reclaim:
            message = ReclaimAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ClaimDelegation:
            message = ClaimDelegationAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ReclaimDelegation:
            message = ReclaimDelegationAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ExtendTicket:
            message = ExtendTicketAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.Relinquish:
            message = RelinquishAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ExtendLease:
            message = ExtendLeaseAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ModifyLease:
            message = ModifyLeaseAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.Close:
            message = CloseAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.UpdateTicket:
            message = UpdateTicketAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.UpdateDelegation:
            message = UpdateDelegationAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.UpdateLease:
            message = UpdateLeaseAvro()
            message.from_dict(value)
        # Management Messages
        elif value['name'] == IMessageAvro.ClaimResources:
            message = ClaimResourcesAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ReclaimResources:
            message = ReclaimResourcesAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.RemoveSlice:
            message = RemoveSliceAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.AddSlice:
            message = AddSliceAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.UpdateSlice:
            message = UpdateSliceAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.RemoveReservation:
            message = RemoveReservationAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.CloseReservations:
            message = CloseReservationsAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.UpdateReservation:
            message = UpdateReservationAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.AddReservation:
            message = AddReservationAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.AddReservations:
            message = AddReservationsAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.DemandReservation:
            message = DemandReservationAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ExtendReservation:
            message = ExtendReservationAvro()
            message.from_dict(value)


        # Get Messages
        elif value['name'] == IMessageAvro.GetSlicesRequest:
            message = GetSlicesRequestAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.GetReservationsRequest:
            message = GetReservationsRequestAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.GetDelegations:
            message = GetDelegationsAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.GetReservationsStateRequest:
            message = GetReservationsStateRequestAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.GetReservationUnitsRequest:
            message = GetReservationUnitsAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.GetUnitRequest:
            message = GetUnitAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.GetPoolInfoRequest:
            message = GetPoolInfoAvro()
            message.from_dict(value)

        # Responses
        elif value['name'] == IMessageAvro.ResultSlice:
            message = ResultSliceAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ResultReservation:
            message = ResultReservationAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ResultDelegation:
            message = ResultDelegationAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ResultReservationState:
            message = ResultReservationStateAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ResultStrings:
            message = ResultStringsAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ResultString:
            message = ResultStringAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ResultUnits:
            message = ResultUnitAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ResultProxy:
            message = ResultProxyAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ResultPool:
            message = ResultPoolInfoAvro()
            message.from_dict(value)
        elif value['name'] == IMessageAvro.ResultActor:
            message = ResultActorAvro()
            message.from_dict(value)
        else:
            self.log_error("Unsupported message: {}".format(value))
            return

        self.handle_message(message=message)

    def handle_message(self, message: IMessageAvro):
        print(message)
        return

    def consume_auto(self):
        """
            Consume records unless shutdown triggered. Uses Kafka's auto commit.
        """
        self.consumer.subscribe(self.topics)

        while self.running:
            try:
                msg = self.consumer.poll(1)

                # There were no messages on the queue, continue polling
                if msg is None:
                    continue

                if msg.error():
                    self.log_error("Consumer error: {}".format(msg.error()))
                    if msg.error().code() == KafkaError._PARTITION_EOF:
                        # End of partition event
                        self.log_error('%% %s [%d] reached end at offset %d\n' % (msg.topic(), msg.partition(), msg.offset()))
                    elif msg.error():
                        self.log_error("Consumer error: {}".format(msg.error()))
                        continue

                self.process_message(msg.topic(), msg.key(), msg.value())
            except SerializerError as e:
                # Report malformed record, discard results, continue polling
                self.log_error("Message deserialization failed {}".format(e))
                continue
            except KeyboardInterrupt:
                break

        self.log_debug("Shutting down consumer..")
        self.consumer.close()

    def consume_sync(self):
        """
            Consume records unless shutdown triggered. Using synchronous commit after a message batch.
        """
        self.consumer.subscribe(self.topics)

        msg_count = 0
        while self.running:
            try:
                msg = self.consumer.poll(1)

                # There were no messages on the queue, continue polling
                if msg is None:
                    continue

                if msg.error():
                    self.log_error("Consumer error: {}".format(msg.error()))
                    if msg.error().code() == KafkaError._PARTITION_EOF:
                        # End of partition event
                        self.log_error('%% %s [%d] reached end at offset %d\n' % (msg.topic(), msg.partition(), msg.offset()))
                    elif msg.error():
                        self.log_error("Consumer error: {}".format(msg.error()))
                        continue

                self.process_message(msg.topic(), msg.key(), msg.value())
                msg_count += 1
                if msg_count % self.batch_size == 0:
                    self.consumer.commit(asynchronous=False)

            except SerializerError as e:
                # Report malformed record, discard results, continue polling
                self.log_error("Message deserialization failed {}".format(e))
                continue
            except KeyboardInterrupt:
                break

        self.log_debug("Shutting down consumer..")
        self.consumer.close()



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric.message_bus.messages.resource_set_avro import ResourceSetAvro
from fabric.message_bus.messages.slice_avro import SliceAvro
from fabric.message_bus.messages.term_avro import TermAvro


class ReservationAvro:
    # Use __slots__ to explicitly declare all data members.
    __slots__ = ["reservation_id", "slice", "resource_set", "term", "sequence"]

    def __init__(self):
        self.reservation_id = None
        self.slice = None
        self.resource_set = None
        self.term = None
        self.sequence = None

    def from_dict(self, value: dict):
        self.reservation_id = value['reservation_id']
        self.sequence = value.get('sequence', None)
        self.slice = SliceAvro()
        self.slice.from_dict(value['slice'])
        self.term = TermAvro()
        self.term.from_dict(value['term'])
        if 'resource_set' in value and value['resource_set'] != "null":
            self.resource_set = ResourceSetAvro()
            self.resource_set.from_dict(value['resource_set'])

    def to_dict(self) -> dict:
        """
            The Avro Python library does not support code generation.
            For this reason we must provide a dict representation of our class for serialization.
        """
        if not self.validate():
            raise Exception("Invalid arguments")

        result = {
            "reservation_id": self.reservation_id,
            "slice": self.slice.to_dict(),
            "term": self.term.to_dict(),
        }
        if self.sequence is not None:
            result['sequence'] = self.sequence
        if self.resource_set is not None:
            result['resource_set'] = self.resource_set.to_dict()
        return result

    def __str__(self):
        return "reservation_id: {} slice: {} term: {} sequence: {} resource_set: {}".format(self.reservation_id,
                                                                                            self.slice,
                                                                                            self.term,
                                                                                            self.sequence,
                                                                                            self.resource_set)

    def __eq__(self, other):
        if not isinstance(other, ReservationAvro):
            return False
        return self.reservation_id == other.reservation_id and self.slice == other.slice and \
               self.term == other.term and self.resource_set == other.resource_set

    def validate(self) -> bool:
        ret_val = True
        if self.reservation_id is None or self.slice is None or self.term is None:
            ret_val = False
        return ret_val